package sun.security.provider.certpath;

import java.security.PublicKey;
import java.security.cert.TrustAnchor;
import java.util.Set;
import java.security.SignatureException;
import java.security.NoSuchAlgorithmException;
import java.security.InvalidKeyException;
import java.security.Signature;
import javax.security.auth.x500.X500Principal;
import java.util.Iterator;
import java.security.GeneralSecurityException;
import java.security.cert.CertificateParsingException;
import sun.security.x509.KeyIdentifier;
import java.security.cert.CertPathValidatorException;
import java.security.cert.X509Certificate;
import java.util.Collections;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.Arrays;
import sun.security.x509.PKIXExtensions;
import java.util.HashMap;
import sun.security.util.DerInputStream;
import java.io.IOException;
import sun.security.util.DerValue;
import sun.misc.HexDumpEncoder;
import java.security.AccessController;
import sun.security.action.GetIntegerAction;
import java.security.cert.Extension;
import java.util.Date;
import sun.security.x509.X509CertImpl;
import java.util.List;
import sun.security.x509.AlgorithmId;
import java.util.Map;
import java.security.cert.CRLReason;
import sun.security.util.ObjectIdentifier;
import sun.security.util.Debug;

public final class OCSPResponse
{
    private static final OCSPResponse.ResponseStatus[] rsvalues;
    private static final Debug debug;
    private static final boolean dump;
    private static final ObjectIdentifier OCSP_BASIC_RESPONSE_OID;
    private static final int CERT_STATUS_GOOD = 0;
    private static final int CERT_STATUS_REVOKED = 1;
    private static final int CERT_STATUS_UNKNOWN = 2;
    private static final int NAME_TAG = 1;
    private static final int KEY_TAG = 2;
    private static final String KP_OCSP_SIGNING_OID = "1.3.6.1.5.5.7.3.9";
    private static final int DEFAULT_MAX_CLOCK_SKEW = 900000;
    private static final int MAX_CLOCK_SKEW;
    private static final CRLReason[] values;
    private final OCSPResponse.ResponseStatus responseStatus;
    private final Map<CertId, OCSPResponse.SingleResponse> singleResponseMap;
    private final AlgorithmId sigAlgId;
    private final byte[] signature;
    private final byte[] tbsResponseData;
    private final byte[] responseNonce;
    private List<X509CertImpl> certs;
    private X509CertImpl signerCert;
    private final ResponderId respId;
    private Date producedAtDate;
    private final Map<String, Extension> responseExtensions;
    
    private static int initializeClockSkew() {
        Integer n = (Integer)AccessController.doPrivileged(new GetIntegerAction("com.sun.security.ocsp.clockSkew"));
        if (n == null || (int)n < 0) {
            return 900000;
        }
        return (int)n * 1000;
    }
    
    public OCSPResponse(byte[] encoding) throws IOException {
        this.signerCert = null;
        this.producedAtDate = null;
        if (OCSPResponse.dump) {
            OCSPResponse.debug.println("OCSPResponse bytes...\n\n" + new HexDumpEncoder().encode(encoding) + "\n");
        }
        DerValue o = new DerValue(encoding);
        if (o.tag != 48) {
            throw new IOException("Bad encoding in OCSP response: expected ASN.1 SEQUENCE tag.");
        }
        DerInputStream data = o.getData();
        int enumerated = data.getEnumerated();
        if (enumerated >= 0 && enumerated < OCSPResponse.rsvalues.length) {
            this.responseStatus = OCSPResponse.rsvalues[enumerated];
            if (OCSPResponse.debug != null) {
                OCSPResponse.debug.println("OCSP response status: " + this.responseStatus);
            }
            if (this.responseStatus != OCSPResponse.ResponseStatus.SUCCESSFUL) {
                this.singleResponseMap = Collections.emptyMap();
                this.certs = new ArrayList();
                this.sigAlgId = null;
                this.signature = null;
                this.tbsResponseData = null;
                this.responseNonce = null;
                this.responseExtensions = Collections.emptyMap();
                this.respId = null;
                return;
            }
            DerValue derValue = data.getDerValue();
            if (!derValue.isContextSpecific((byte)0)) {
                throw new IOException("Bad encoding in responseBytes element of OCSP response: expected ASN.1 context specific tag 0.");
            }
            DerValue derValue2 = derValue.data.getDerValue();
            if (derValue2.tag != 48) {
                throw new IOException("Bad encoding in responseBytes element of OCSP response: expected ASN.1 SEQUENCE tag.");
            }
            DerInputStream data2 = derValue2.data;
            ObjectIdentifier oid = data2.getOID();
            if (oid.equals(OCSPResponse.OCSP_BASIC_RESPONSE_OID)) {
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("OCSP response type: basic");
                }
                DerValue[] sequence = new DerInputStream(data2.getOctetString()).getSequence(3);
                if (sequence.length < 3) {
                    throw new IOException("Unexpected BasicOCSPResponse value");
                }
                DerValue derValue3 = sequence[0];
                this.tbsResponseData = sequence[0].toByteArray();
                if (derValue3.tag != 48) {
                    throw new IOException("Bad encoding in tbsResponseData element of OCSP response: expected ASN.1 SEQUENCE tag.");
                }
                DerInputStream data3 = derValue3.data;
                DerValue derValue4 = data3.getDerValue();
                if (derValue4.isContextSpecific((byte)0) && derValue4.isConstructed() && derValue4.isContextSpecific()) {
                    DerValue derValue5 = derValue4.data.getDerValue();
                    derValue5.getInteger();
                    if (derValue5.data.available() != 0) {
                        throw new IOException("Bad encoding in version  element of OCSP response: bad format");
                    }
                    derValue4 = data3.getDerValue();
                }
                this.respId = new ResponderId(derValue4.toByteArray());
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("Responder ID: " + this.respId);
                }
                this.producedAtDate = data3.getDerValue().getGeneralizedTime();
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("OCSP response produced at: " + this.producedAtDate);
                }
                DerValue[] sequence2 = data3.getSequence(1);
                this.singleResponseMap = new HashMap(sequence2.length);
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("OCSP number of SingleResponses: " + sequence2.length);
                }
                for (int length = sequence2.length, i = 0; i < length; ++i) {
                    OCSPResponse.SingleResponse o2 = new OCSPResponse.SingleResponse(sequence2[i]);
                    this.singleResponseMap.put(o2.getCertId(), o2);
                }
                HashMap extensions = new HashMap();
                if (data3.available() > 0) {
                    DerValue derValue6 = data3.getDerValue();
                    if (derValue6.isContextSpecific((byte)1)) {
                        extensions = parseExtensions(derValue6);
                    }
                }
                this.responseExtensions = extensions;
                sun.security.x509.Extension extension = (sun.security.x509.Extension)extensions.get(PKIXExtensions.OCSPNonce_Id.toString());
                this.responseNonce = ((extension != null) ? extension.getExtensionValue() : null);
                if (OCSPResponse.debug != null && this.responseNonce != null) {
                    OCSPResponse.debug.println("Response nonce: " + Arrays.toString(this.responseNonce));
                }
                this.sigAlgId = AlgorithmId.parse(sequence[1]);
                this.signature = sequence[2].getBitString();
                if (sequence.length > 3) {
                    DerValue derValue7 = sequence[3];
                    if (!derValue7.isContextSpecific((byte)0)) {
                        throw new IOException("Bad encoding in certs element of OCSP response: expected ASN.1 context specific tag 0.");
                    }
                    DerValue[] sequence3 = derValue7.getData().getSequence(3);
                    this.certs = new ArrayList(sequence3.length);
                    try {
                        for (int j = 0; j < sequence3.length; ++j) {
                            X509CertImpl o3 = new X509CertImpl(sequence3[j].toByteArray());
                            this.certs.add(o3);
                            if (OCSPResponse.debug != null) {
                                OCSPResponse.debug.println("OCSP response cert #" + (j + 1) + ": " + o3.getSubjectX500Principal());
                            }
                        }
                    }
                    catch (CertificateException cause) {
                        throw new IOException("Bad encoding in X509 Certificate", cause);
                    }
                }
                else {
                    this.certs = new ArrayList();
                }
                return;
            }
            else {
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("OCSP response type: " + oid);
                }
                throw new IOException("Unsupported OCSP response type: " + oid);
            }
        }
        else {
            throw new IOException("Unknown OCSPResponse status: " + enumerated);
        }
    }
    
    void verify(List<CertId> list, OCSPResponse.IssuerInfo issuerInfo, X509Certificate x509Certificate, Date date, byte[] a, String s) throws CertPathValidatorException {
        switch (this.responseStatus) {
            case SUCCESSFUL: {
                for (CertId obj : list) {
                    OCSPResponse.SingleResponse singleResponse = this.getSingleResponse(obj);
                    if (singleResponse == null) {
                        if (OCSPResponse.debug != null) {
                            OCSPResponse.debug.println("No response found for CertId: " + obj);
                        }
                        throw new CertPathValidatorException("OCSP response does not include a response for a certificate supplied in the OCSP request");
                    }
                    else if (OCSPResponse.debug != null) {
                        OCSPResponse.debug.println("Status of certificate (with serial number " + obj.getSerialNumber() + ") is: " + singleResponse.getCertStatus());
                    }
                    else {
                        continue;
                    }
                }
                if (this.signerCert == null) {
                    try {
                        if (issuerInfo.getCertificate() != null) {
                            this.certs.add(X509CertImpl.toImpl(issuerInfo.getCertificate()));
                        }
                        if (x509Certificate != null) {
                            this.certs.add(X509CertImpl.toImpl(x509Certificate));
                        }
                    }
                    catch (CertificateException cause) {
                        throw new CertPathValidatorException("Invalid issuer or trusted responder certificate", cause);
                    }
                    if (this.respId.getType() == ResponderId.Type.BY_NAME) {
                        X500Principal responderName = this.respId.getResponderName();
                        for (X509CertImpl signerCert : this.certs) {
                            if (signerCert.getSubjectX500Principal().equals(responderName)) {
                                this.signerCert = signerCert;
                                break;
                            }
                            continue;
                        }
                    }
                    else if (this.respId.getType() == ResponderId.Type.BY_KEY) {
                        KeyIdentifier keyIdentifier = this.respId.getKeyIdentifier();
                        for (X509CertImpl x509CertImpl : this.certs) {
                            KeyIdentifier subjectKeyId = x509CertImpl.getSubjectKeyId();
                            if (subjectKeyId != null && keyIdentifier.equals(subjectKeyId)) {
                                this.signerCert = x509CertImpl;
                                break;
                            }
                            try {
                                subjectKeyId = new KeyIdentifier(x509CertImpl.getPublicKey());
                            }
                            catch (IOException ex) {}
                            if (keyIdentifier.equals(subjectKeyId)) {
                                this.signerCert = x509CertImpl;
                                break;
                            }
                            continue;
                        }
                    }
                }
                if (this.signerCert != null) {
                    if (this.signerCert.getSubjectX500Principal().equals(issuerInfo.getName()) && this.signerCert.getPublicKey().equals(issuerInfo.getPublicKey())) {
                        if (OCSPResponse.debug != null) {
                            OCSPResponse.debug.println("OCSP response is signed by the target's Issuing CA");
                        }
                    }
                    else if (this.signerCert.equals(x509Certificate)) {
                        if (OCSPResponse.debug != null) {
                            OCSPResponse.debug.println("OCSP response is signed by a Trusted Responder");
                        }
                    }
                    else if (this.signerCert.getIssuerX500Principal().equals(issuerInfo.getName())) {
                        try {
                            List<String> extendedKeyUsage = this.signerCert.getExtendedKeyUsage();
                            if (extendedKeyUsage == null || !extendedKeyUsage.contains("1.3.6.1.5.5.7.3.9")) {
                                throw new CertPathValidatorException("Responder's certificate not valid for signing OCSP responses");
                            }
                        }
                        catch (CertificateParsingException cause2) {
                            throw new CertPathValidatorException("Responder's certificate not valid for signing OCSP responses", cause2);
                        }
                        AlgorithmChecker o = new AlgorithmChecker(issuerInfo.getAnchor(), date, s);
                        o.init(false);
                        o.check(this.signerCert, Collections.emptySet());
                        try {
                            if (date == null) {
                                this.signerCert.checkValidity();
                            }
                            else {
                                this.signerCert.checkValidity(date);
                            }
                        }
                        catch (CertificateException cause3) {
                            throw new CertPathValidatorException("Responder's certificate not within the validity period", cause3);
                        }
                        if (this.signerCert.getExtension(PKIXExtensions.OCSPNoCheck_Id) != null && OCSPResponse.debug != null) {
                            OCSPResponse.debug.println("Responder's certificate includes the extension id-pkix-ocsp-nocheck.");
                        }
                        try {
                            this.signerCert.verify(issuerInfo.getPublicKey());
                            if (OCSPResponse.debug != null) {
                                OCSPResponse.debug.println("OCSP response is signed by an Authorized Responder");
                            }
                        }
                        catch (GeneralSecurityException ex2) {
                            this.signerCert = null;
                        }
                    }
                    else {
                        throw new CertPathValidatorException("Responder's certificate is not authorized to sign OCSP responses");
                    }
                }
                if (this.signerCert != null) {
                    AlgorithmChecker.check(this.signerCert.getPublicKey(), this.sigAlgId, s);
                    if (!this.verifySignature(this.signerCert)) {
                        throw new CertPathValidatorException("Error verifying OCSP Response's signature");
                    }
                    if (a != null && this.responseNonce != null && !Arrays.equals(a, this.responseNonce)) {
                        throw new CertPathValidatorException("Nonces don't match");
                    }
                    long date2 = (date == null) ? System.currentTimeMillis() : date.getTime();
                    Date o2 = new Date(date2 + (long)OCSPResponse.MAX_CLOCK_SKEW);
                    Date o3 = new Date(date2 - (long)OCSPResponse.MAX_CLOCK_SKEW);
                    for (OCSPResponse.SingleResponse singleResponse2 : this.singleResponseMap.values()) {
                        if (OCSPResponse.debug != null) {
                            String string = "";
                            if (singleResponse2.nextUpdate != null) {
                                string = " until " + singleResponse2.nextUpdate;
                            }
                            OCSPResponse.debug.println("OCSP response validity interval is from " + singleResponse2.thisUpdate + string);
                            OCSPResponse.debug.println("Checking validity of OCSP response on: " + new Date(date2));
                        }
                        if (o2.before(singleResponse2.thisUpdate) || o3.after((singleResponse2.nextUpdate != null) ? singleResponse2.nextUpdate : singleResponse2.thisUpdate)) {
                            throw new CertPathValidatorException("Response is unreliable: its validity interval is out-of-date");
                        }
                        continue;
                    }
                    return;
                }
                else {
                    throw new CertPathValidatorException("Unable to verify OCSP Response's signature");
                }
            }
            case TRY_LATER:
            case INTERNAL_ERROR: {
                throw new CertPathValidatorException("OCSP response error: " + this.responseStatus, null, null, -1, CertPathValidatorException.BasicReason.UNDETERMINED_REVOCATION_STATUS);
            }
            default: {
                throw new CertPathValidatorException("OCSP response error: " + this.responseStatus);
            }
        }
    }
    
    public OCSPResponse.ResponseStatus getResponseStatus() {
        return this.responseStatus;
    }
    
    private boolean verifySignature(X509Certificate x509Certificate) throws CertPathValidatorException {
        try {
            Signature instance = Signature.getInstance(this.sigAlgId.getName());
            instance.initVerify(x509Certificate.getPublicKey());
            instance.update(this.tbsResponseData);
            if (instance.verify(this.signature)) {
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("Verified signature of OCSP Response");
                }
                return true;
            }
            else {
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("Error verifying signature of OCSP Response");
                }
                return false;
            }
        }
        catch (InvalidKeyException | NoSuchAlgorithmException | SignatureException cause) {
            throw new CertPathValidatorException(cause);
        }
    }
    
    public OCSPResponse.SingleResponse getSingleResponse(CertId certId) {
        return (OCSPResponse.SingleResponse)this.singleResponseMap.get(certId);
    }
    
    public Set<CertId> getCertIds() {
        return Collections.unmodifiableSet(this.singleResponseMap.keySet());
    }
    
    X509Certificate getSignerCertificate() {
        return this.signerCert;
    }
    
    public ResponderId getResponderId() {
        return this.respId;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("OCSP Response:\n");
        sb.append("Response Status: ").append(this.responseStatus).append("\n");
        sb.append("Responder ID: ").append(this.respId).append("\n");
        sb.append("Produced at: ").append(this.producedAtDate).append("\n");
        int size = this.singleResponseMap.size();
        sb.append(size).append((size == 1) ? " response:\n" : " responses:\n");
        Iterator<OCSPResponse.SingleResponse> iterator = this.singleResponseMap.values().iterator();
        while (iterator.hasNext()) {
            sb.append((Object)iterator.next()).append("\n");
        }
        if (this.responseExtensions != null && this.responseExtensions.size() > 0) {
            int size2 = this.responseExtensions.size();
            sb.append(size2).append((size2 == 1) ? " extension:\n" : " extensions:\n");
            Iterator<String> iterator2 = this.responseExtensions.keySet().iterator();
            while (iterator2.hasNext()) {
                sb.append(this.responseExtensions.get((Object)iterator2.next())).append("\n");
            }
        }
        return sb.toString();
    }
    
    private static Map<String, Extension> parseExtensions(DerValue derValue) throws IOException {
        DerValue[] sequence = derValue.data.getSequence(3);
        HashMap hashMap = new HashMap(sequence.length);
        int length = sequence.length;
        int i = 0;
        while (i < length) {
            Extension obj = new sun.security.x509.Extension(sequence[i]);
            if (OCSPResponse.debug != null) {
                OCSPResponse.debug.println("Extension: " + obj);
            }
            if (obj.isCritical()) {
                throw new IOException("Unsupported OCSP critical extension: " + obj.getExtensionId());
            }
            hashMap.put(obj.getId(), obj);
            ++i;
        }
        return hashMap;
    }
    
    static Map access$500(DerValue derValue) throws IOException {
        return parseExtensions(derValue);
    }
    
    static {
        rsvalues = OCSPResponse.ResponseStatus.values();
        debug = Debug.getInstance("certpath");
        dump = (OCSPResponse.debug != null && Debug.isOn("ocsp"));
        OCSP_BASIC_RESPONSE_OID = ObjectIdentifier.newInternal(new int[] { 1, 3, 6, 1, 5, 5, 7, 48, 1, 1 });
        MAX_CLOCK_SKEW = initializeClockSkew();
        values = CRLReason.values();
    }
    
    public enum ResponseStatus
    {
        SUCCESSFUL, 
        MALFORMED_REQUEST, 
        INTERNAL_ERROR, 
        TRY_LATER, 
        UNUSED, 
        SIG_REQUIRED, 
        UNAUTHORIZED;
    }
    
    public static final class SingleResponse implements OCSP.RevocationStatus
    {
        private final CertId certId;
        private final OCSP.RevocationStatus.CertStatus certStatus;
        private final Date thisUpdate;
        private final Date nextUpdate;
        private final Date revocationTime;
        private final CRLReason revocationReason;
        private final Map<String, Extension> singleExtensions;
        
        private SingleResponse(DerValue derValue) throws IOException {
            if (derValue.tag != 48) {
                throw new IOException("Bad ASN.1 encoding in SingleResponse");
            }
            DerInputStream data = derValue.data;
            this.certId = new CertId(data.getDerValue().data);
            DerValue derValue2 = data.getDerValue();
            int n = (short)(byte)(derValue2.tag & 0x1F);
            if (n == 1) {
                this.certStatus = OCSP.RevocationStatus.CertStatus.REVOKED;
                this.revocationTime = derValue2.data.getGeneralizedTime();
                if (derValue2.data.available() != 0) {
                    DerValue derValue3 = derValue2.data.getDerValue();
                    if ((short)(byte)(derValue3.tag & 0x1F) == 0) {
                        int enumerated = derValue3.data.getEnumerated();
                        if (enumerated >= 0 && enumerated < OCSPResponse.values.length) {
                            this.revocationReason = OCSPResponse.values[enumerated];
                        }
                        else {
                            this.revocationReason = CRLReason.UNSPECIFIED;
                        }
                    }
                    else {
                        this.revocationReason = CRLReason.UNSPECIFIED;
                    }
                }
                else {
                    this.revocationReason = CRLReason.UNSPECIFIED;
                }
                if (OCSPResponse.debug != null) {
                    OCSPResponse.debug.println("Revocation time: " + this.revocationTime);
                    OCSPResponse.debug.println("Revocation reason: " + this.revocationReason);
                }
            }
            else {
                this.revocationTime = null;
                this.revocationReason = null;
                if (n == 0) {
                    this.certStatus = OCSP.RevocationStatus.CertStatus.GOOD;
                }
                else if (n == 2) {
                    this.certStatus = OCSP.RevocationStatus.CertStatus.UNKNOWN;
                }
                else {
                    throw new IOException("Invalid certificate status");
                }
            }
            this.thisUpdate = data.getGeneralizedTime();
            if (OCSPResponse.debug != null) {
                OCSPResponse.debug.println("thisUpdate: " + this.thisUpdate);
            }
            Date generalizedTime = null;
            Map<Object, Object> access$500 = null;
            if (data.available() > 0) {
                DerValue derValue4 = data.getDerValue();
                if (derValue4.isContextSpecific((byte)0)) {
                    generalizedTime = derValue4.data.getGeneralizedTime();
                    if (OCSPResponse.debug != null) {
                        OCSPResponse.debug.println("nextUpdate: " + generalizedTime);
                    }
                    derValue4 = ((data.available() > 0) ? data.getDerValue() : null);
                }
                if (derValue4 != null) {
                    if (derValue4.isContextSpecific((byte)1)) {
                        access$500 = OCSPResponse.access$500(derValue4);
                        if (data.available() > 0) {
                            throw new IOException(data.available() + " bytes of additional data in singleResponse");
                        }
                    }
                    else {
                        throw new IOException("Unsupported singleResponse item, tag = " + String.format("%02X", new Object[] { Byte.valueOf(derValue4.tag) }));
                    }
                }
            }
            this.nextUpdate = generalizedTime;
            this.singleExtensions = ((access$500 != null) ? access$500 : Collections.emptyMap());
            if (OCSPResponse.debug != null) {
                Iterator<Extension> iterator = this.singleExtensions.values().iterator();
                while (iterator.hasNext()) {
                    OCSPResponse.debug.println("singleExtension: " + (Object)iterator.next());
                }
            }
            return;
        }
        
        @Override
        public OCSP.RevocationStatus.CertStatus getCertStatus() {
            return this.certStatus;
        }
        
        public CertId getCertId() {
            return this.certId;
        }
        
        public Date getThisUpdate() {
            return (this.thisUpdate != null) ? ((Date)this.thisUpdate.clone()) : null;
        }
        
        public Date getNextUpdate() {
            return (this.nextUpdate != null) ? ((Date)this.nextUpdate.clone()) : null;
        }
        
        @Override
        public Date getRevocationTime() {
            return (this.revocationTime != null) ? ((Date)this.revocationTime.clone()) : null;
        }
        
        @Override
        public CRLReason getRevocationReason() {
            return this.revocationReason;
        }
        
        @Override
        public Map<String, Extension> getSingleExtensions() {
            return Collections.unmodifiableMap(this.singleExtensions);
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append("SingleResponse:\n");
            sb.append(this.certId);
            sb.append("\nCertStatus: ").append(this.certStatus).append("\n");
            if (this.certStatus == OCSP.RevocationStatus.CertStatus.REVOKED) {
                sb.append("revocationTime is ");
                sb.append(this.revocationTime).append("\n");
                sb.append("revocationReason is ");
                sb.append(this.revocationReason).append("\n");
            }
            sb.append("thisUpdate is ").append(this.thisUpdate).append("\n");
            if (this.nextUpdate != null) {
                sb.append("nextUpdate is ").append(this.nextUpdate).append("\n");
            }
            for (Extension extension : this.singleExtensions.values()) {
                sb.append("singleExtension: ");
                sb.append(extension.toString()).append("\n");
            }
            return sb.toString();
        }
    }
    
    static final class IssuerInfo
    {
        private final TrustAnchor anchor;
        private final X509Certificate certificate;
        private final X500Principal name;
        private final PublicKey pubKey;
        
        IssuerInfo(TrustAnchor trustAnchor) {
            this(trustAnchor, (trustAnchor != null) ? trustAnchor.getTrustedCert() : null);
        }
        
        IssuerInfo(X509Certificate x509Certificate) {
            this(null, x509Certificate);
        }
        
        IssuerInfo(TrustAnchor anchor, X509Certificate certificate) {
            if (anchor == null && certificate == null) {
                throw new NullPointerException("TrustAnchor and issuerCert cannot be null");
            }
            this.anchor = anchor;
            if (certificate != null) {
                this.name = certificate.getSubjectX500Principal();
                this.pubKey = certificate.getPublicKey();
                this.certificate = certificate;
            }
            else {
                this.name = anchor.getCA();
                this.pubKey = anchor.getCAPublicKey();
                this.certificate = anchor.getTrustedCert();
            }
            return;
        }
        
        X509Certificate getCertificate() {
            return this.certificate;
        }
        
        X500Principal getName() {
            return this.name;
        }
        
        PublicKey getPublicKey() {
            return this.pubKey;
        }
        
        TrustAnchor getAnchor() {
            return this.anchor;
        }
        
        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append("Issuer Info:\n");
            sb.append("Name: ").append(this.name.toString()).append("\n");
            sb.append("Public Key:\n").append(this.pubKey.toString()).append("\n");
            return sb.toString();
        }
    }
}
